package eu.clarin.cmdi.mdservice.action;

import java.io.IOException;
import java.io.InputStream;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.jar.*;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;
import org.apache.struts2.interceptor.ServletRequestAware;
import org.w3c.dom.Document;

import com.opensymphony.xwork2.ActionSupport;
import com.sun.xml.internal.ws.util.StringUtils;

import eu.clarin.cmdi.mdservice.internal.CQLParseException;
import eu.clarin.cmdi.mdservice.internal.Cache;
import eu.clarin.cmdi.mdservice.internal.MDTransformer;
import eu.clarin.cmdi.mdservice.internal.NoStylesheetException;
import eu.clarin.cmdi.mdservice.internal.Utils;
import eu.clarin.cmdi.mdservice.model.Diagnostic;
import eu.clarin.cmdi.mdservice.model.Diagnostics;
import eu.clarin.cmdi.mdservice.model.WorkspaceProfile;
import eu.clarin.cmdi.mdservice.proxy.ProxyInterface;



/**
 * Main Struts2 controller (ie implemenets the struts' execute()-method), all other Actions are derived from.   
 * Defines the methods used for retrieving the data 
 * and provides basic implementation where possible.
 * Also responsible for caching and transformation of the result. 
 *  
 * @author vronk
 */
public class GenericAction extends ActionSupport 
implements ServletRequestAware 
{
	
	public static Logger log = Logger.getLogger("GenericAction");
	private static final long serialVersionUID = 1L;
	
	private Diagnostics diagnostics;

	private String actionkey = "generic";
	private String q;
	private String format = "xml";
	
	private String actionContentType;
	private HttpServletRequest request; 
	private long duration = 0;
	private InputStream resultStream;
	private InputStream sourceStream;
	
	private Map<String,String[]> params;
	
	private ProxyInterface target_proxy;
		
	 /**
	  * Gets the target proxy.
	  * 		
	  * @return target proxy
	  */
	public ProxyInterface getTargetProxy() {
		return target_proxy;
	}

	/**
	 * Sets the particular proxy and its  source action.
	 * 
	 * @param proxy - interface to be set
	 */
	public void setTargetProxy(ProxyInterface proxy) {
		target_proxy = proxy;
		target_proxy.setSourceAction(this);
	}

	/**
	 * Empty implementation, subclasses have to decide themselves, which proxy to take.
	 */	
	public void setTargetProxy() {
	}

	
	/**
	 * Gets the diagnostics.
	 * 
	 * @return diagnostics
	 */
	public Diagnostics getDiagnostics(){
		 if (diagnostics == null){
			 diagnostics  = new Diagnostics();
		 }
		 return this.diagnostics;
	}
	 
	/**
	 * Sets the diagnostics.
	 * 
	 * @param diagnostics
	 */
	public void setDiagnostics(Diagnostics diagnostics){
		 this.diagnostics = diagnostics;
	}
	 
	/**
	 * Gets string variable q.
	 * 
	 * @return 
	 */
	public String getQ() {
			return q;
	}

	/**
	 * Sets the variable q.
	 * 
	 * @param q
	 */
	public void setQ(String q) {
			this.q = q;
	}
	
	/**
	 * Sets the variable format. Used by Struts in case the format is part of the request path.
	 * In the case the format is part of the request parameters, it is read by servlet request.
	 * 
	 * @param format
	 */
	public void setFormat(String format) {
			this.format = format;
	}
	
	
	public String getActionkey() {
			return actionkey;
		}

	public void setActionkey(String actionKey) {
			actionkey = actionKey;
		}
	
	/**
	 * Gets the scanClause parameter from the parameter map.
	 * 
	 * @return
	 */
	public String getScanClause() {
		 return getParam("scanClause");
	}
	
	/**
	 * Gets the recordPacking parameter from the parameter map.
	 * 
	 * @return
	 */
	public String getRecordPacking() {
		 return getParam("recordPacking");
	}
	
	public String getQuery() {
		 return getParam("query");
	}
	
	public String getStartRecord() {
		 return getParam("startRecord");
	}
	
	public String getMaximumRecords() {
		 return getParam("maximumRecords");
	}
	
	public String getFormat() {
			return getParam("format");
	}
	
	public String getCache() {
			return getParam("cache");
	}
	
	public String getRepository() {
			return getParam("repository");
	}
	
	public String getLang() {
			return getParam("lang");
	}
	
	public String getCollections() {
			return getParam("collections");
	}
	
	public String getColumns() {
			return getParam("columns");
	}
	
	public String getMaxdepth() {
		 return getParam("maxdepth");
	}
	 
	
	public GenericAction() {
		//configure
		Utils.loadConfig("mdservice", "mdservice.properties", this.getClass().getClassLoader());

	}
	/**
	 * Is used in struts.xml to dynamically set the mime-type of the result, depending on the format-parameter
	 * 
	 * @return
	 */
	public String getActionContentType() {
		if (this.getFormat().toLowerCase().startsWith("html")) { // && !getDiagnostics().Exists()) {
			this.actionContentType = "text/html";		
			//this.actionContentType = "application/xhtml+xml";
		} else {
			this.actionContentType = "text/xml";
		}
		
		return actionContentType;
	}

	public void setActionContentType(String actionContentType) {
		this.actionContentType = actionContentType;
	}


	public String getFullFormat() {		
		return actionkey + "2" + getFormat();
	}
	
	/**
	 * Needed to acquire the original ServletRequest 
	 * (want to access the raw params - in loadParams())
	 * @return
	 */
	@Override
	public void setServletRequest(HttpServletRequest arg0) {
		request = arg0;
	}

	public HttpServletRequest getServletRequest() {
		return request;	
	}

	/**
	 * The stream holding the resulting data to be sent back to the user as response.
	 * @return
	 */
	public InputStream getResultStream() {
	        return resultStream;
    }
	
	public void setResultStream(InputStream _resultStream){
		resultStream = _resultStream;
	}
	
	public void setSourceStream(InputStream _sourceStream){
		sourceStream = _sourceStream;
	}
	
	 
	/**
	 * Get the sourceStream from the target proxy.	 
	 * @return
	 * @throws IOException
	 * @throws NoStylesheetException
	 * @throws CQLParseException 
	 */
	public InputStream getSourceStream() throws IOException, NoStylesheetException, CQLParseException {		
			return getTargetProxy().getSourceStream(); 
	}
		
	/**
	 * Add parameter into local parameter map.
	 * 
	 * @param key - parameter key
	 * @param value - parameter value
	 */
	public void addParam(String key, String value){
		String[] sarr = new String[1];
		sarr[0] = value;
		params.put(key, sarr);	
	}
	
	/**
	 * Set parameter in local parameter map.
	 * 
	 * @param key - parameter key
	 * @param value - parameter value
	 */
	public void setParam(String key, String value){
		addParam(key, value);	
	}
	
	/**
	 * Reads in all the parameters of the request into an internal structure.
	 * the Map is not Map&lt;String,String&gt;, but rather Map&lt;String,String[]&gt;, 
	 * ie the value is an array of Strings! <a href="http://download.oracle.com/docs/cd/E17802_01/webservices/webservices/docs/1.5/api/javax/servlet/ServletRequest.html#getParameterMap%28%29" >javadocs:ServletRequest.getParameterMap()</a>
	 * 
	 * This shall allow more flexibility in using the parameters in the subclasses (and less getter/setter stuff)
	 * at the cost of circumventing the Struts-way.
	 * 
	 * 
	 * The params are completed to fulfill sru params standard an also the params needed by XSL transformation 
	 * are added.
	 * 
	 *  SRU PARAMS:
     * version
     * operation
     * query
     * scanClause
     * startRecord
     * maximumRecords
     * recordPacking
     * recordSchema
     * recordXPath
     * resultSetTTL
     * sortKeys
     * stylesheet
     * x-cmd-repository
     * x-cmd-collections
     * x-cmd-columns
     * x-cmd-lang
     * x-cmd-format
     * x-cmd-cache
     * x-cmd-maxdepth
	 */
	@SuppressWarnings("unchecked")
	protected void loadParams() {		
		params = new HashMap(getServletRequest().getParameterMap());	

		//setDefaultParams();

		// set defaults
		if ( params.get("version") == null){
			addParam("version","1.2");
		}
		if ( params.get("operation") == null){
			addParam("operation","searchRetrieve");
		}
		//q=query			
		if (getQ() != null) {
			addParam("q",getQ());
			addParam("query",getQ());
		} else {
			addParam("q",this.getParam("query"));
		}
		
		if ( params.get("startRecord") == null){
			addParam("startRecord","1");
		}
		if ( params.get("maximumRecords") == null){
			addParam("maximumRecords","50");
		}
		
		if ( params.get("x-cmd-maxdepth") == null){
			if (params.get("maxdepth") == null){
				addParam("maxdepth","2");
			} 
		} else {
			addParam("maxdepth",params.get("x-cmd-maxdepth")[0]);
		}
		
		if ( params.get("x-cmd-repository") == null){
			if (params.get("repository") == null){
				addParam("repository",WorkspaceProfile.getRepositoryByIndex(0));
			} 
		} else {
			addParam("repository",params.get("x-cmd-repository")[0]);
		}
		
		if ( params.get("x-cmd-cache") == null){
			if (params.get("cache") == null){
				addParam("cache",Cache.SKIP);
			} 
		} else {
			addParam("cache",params.get("x-cmd-cache")[0]);
		}
		
		if ( params.get("x-cmd-lang") == null){
			if (params.get("lang") == null){
				addParam("lang","en");
			} 
		} else {
			addParam("lang",params.get("x-cmd-lang")[0]);
		}
		
		if ( params.get("x-cmd-format") == null){
			if (params.get("format") == null){
				if (this.format == null) {
					addParam("format","xml");
				} else if (this.format.equals("")){
					addParam("format","xml");
				} else {
					addParam("format",this.format);
				}
			} 
		} else {
			addParam("format",params.get("x-cmd-format")[0]);
		}

		addParam("fullformat",getFullFormat());
	}
        
	protected void setDefaultParams(){
		//q=query	
		
	
		if (getQ() != null) {
			addParam("q",getQ());
			addParam("query",getQ());
		} else {
			addParam("q",this.getParam("query"));
		}
		
		if (this.format != null) {
			addParam("format",this.format);
		} 
	
		if (getParams().get("repository") == null){
			addParam("repository",WorkspaceProfile.getRepositoryByIndex(0));
		} 
	}
	/**
	 * Gets the local parameter map.
	 * 
	 * @return
	 */
	public Map<String,String[]> getParams() {		
		return params;
	}

	
	/**
	 * This is for simplified access to the the values of the request-parameters 
	 * They are stored in the parameters-map as a String-array, 
	 * but in most(?) situations, we expect just a simple string. 
	 * @param key
	 * @return
	 */
	public String getParam(String key) {
		String v = "";
		if (!(params.get(key)==null)) v=(String)params.get(key)[0];
		return v;
	}

	public String[] getParamArray(String key) {
		String[] a = null;
		if (!(params.get(key)==null)) a=(String[])params.get(key);
		return a;
	}

	/**
	 * Constructs an unambiguous key for the request (encoding all the parameters).
	 * This is used as identifier for caching .
	 * 
	 * @return key unambiguously encoding the request
	 */
	
	 public String getRequestKey() {
			String key="";
			if (getActionkey()!=null) {
				key += getActionkey() + "//-" ;
			}else {
				key +="//-" ;
			}
			if (getQuery()!=null) {
				key += getQuery() + "//-" ;
			} else {
				key +="//-" ;
			}
			if (getCollections()!=null) {
				key += getCollections() + "//-";
			} else {
				key +="//-" ;
			}
			if (this.getStartRecord()!=null) {
				key += getStartRecord() + "//-";
			}
				else{
					key +="//-" ;
			}
			if (getMaximumRecords()!=null) {
				key += getMaximumRecords() + "//-";
			}
			else{
				key +="//-" ;
			}
			
			key += getRepository()  + "//-";
			key += getMaxdepth()  + "//-";
			
			if (getLang()!=null) {
				key += getLang() + "//-";
			}else{
				key +="//-" ;
			}			

		return key;
	}
	
	 
	/**
	 * Adds the duration value of the action execution to the request key.
	 * 
	 * @return
	 */
	public String addDurationKey(){
		String req_key = getRequestKey();
	
		Double duration_d;

	    duration_d = (double)duration; 
	    duration_d = duration_d/1000.0;
		req_key += duration_d + "//-";
		return req_key;
	}
	
	
	public String getTargetRequestParams(){
		return "";
	}
	
	public void setRequestProperties(URLConnection urlConnection){
		
	}

	/**
	 * This is the work-horse function.
	 * It does more things: 
	 * a) Loads the request parameters.
	 * b) Creates the concerned proxy object.
	 * c) Checks if the source data is already in cache (based on cache-key that is constructed from the request parameters).
	 * d) If diagnostics occurred  during function, the diagnostics data will be  added into source data. 
	 * e) If format-parameter provided, invokes the transformation.
	 *  
	 * 
	 * At the end the data to be returned as result is contained in the resultStream
	 * If format is xml, then the data from sourceStream is passed further as resultStream. 
	 * 
	 * If exception occurred within prepare function , it will be caught and written 
	 * as diagnostics system error.
	 *    
	 */
	public void prepare()  {
			
		try{
			
			loadParams();
			
			setTargetProxy();
			if (getTargetProxy() == null) return;			
			getTargetProxy().checkParams();
			
			// Do not continue if something is fatal.
			if (getDiagnostics().Exists()) return;
			
			String req_key = getRequestKey();
			log.debug("request_key: " +  req_key); 
			log.debug("params.size: " +  params.size());
			log.debug(getActionkey() + ".targetURL: " + getTargetProxy().getTargetRequest() + " .format:" + getFullFormat());
			
			//Caching
		
			if (getCache().equals(Cache.SKIP)) {
				
				sourceStream = getSourceStream();
				
			} else { 
				if (getCache().equals(Cache.USE)) {
						sourceStream = Cache.getCache().getFromCache(req_key);
				}
				if (sourceStream == null) { // either not in cache or cache_flag=refresh
					long startMillis = System.currentTimeMillis();
					sourceStream = getSourceStream();
					duration = System.currentTimeMillis() - startMillis;
					req_key = addDurationKey();
					String xcid = Cache.getCache().putInCache(req_key,sourceStream);			
					log.debug("putting in cache: " + req_key + ", " + xcid);			
					sourceStream = Cache.getCache().getFromCache(req_key);
				} else {
					log.debug("reading from cache: " + req_key);
				} 
			}
			
			// append diagnostics to sourcestream
			if (getDiagnostics().Exists()){
				if (sourceStream == null) return;
				
				Document doc = Utils.stream2Document(sourceStream);
				//TODO tagname by class
				doc = Utils.append2Document(doc, "searchRetrieveResponse",  getDiagnostics().buildXMLDocument());
				this.setSourceStream(Utils.document2Stream(doc));
			}
			
			log.debug("format: " + getFormat() );
			if (getFormat().equals("xml")) {			
				resultStream = sourceStream;	
			}else {
				MDTransformer transformer = new MDTransformer();
				transformer.configure(Utils.getConfig(), this.getClass().getClassLoader());
				// set URL as srcFile (for MDTransformer to pass to xsl-scripts)
				// TODO: WHY??
				transformer.setSrcFile(getTargetProxy().getTargetRequest());
				transformer.setParams(getParams());
				// here the transformation is invoked
				resultStream = transformer.transformXML(sourceStream);
			}

		}catch (Exception e){
			log.error(Utils.errorMessage(e));
			//TODO add exception to diagnostic ?
			getDiagnostics().Add(Diagnostic.SYSTEM_ERROR, Utils.errorMessage(e));
			return;
		}
		
		log.debug(getActionkey() + " success:" + (resultStream!=null));
	}

	/**
	 *  Default Action method that gets called by Struts. Everything interesting happens in prepare()
	 *  In case an error occurred in prepare and the result stream is not returned, the diagnostics will be returned
	 *  as result.
	 *  
	 *  @throws Exception  This is handled by struts (is mapped to some result in struts.xml based on the Exception-Class).
	 */
	public String execute() throws Exception {
		
        prepare();
		
		if (resultStream == null) {
			// diagnostics to resultstream
			if (getDiagnostics().Exists()){
				this.setResultStream( Utils.document2Stream(getDiagnostics().buildXMLDocument()));
				return SUCCESS;
			}
			return ERROR;	
		} else {
			return SUCCESS;
		}		
	}


}