package eu.clarin.cmdi.mdservice.action;

import java.net.URL;
import java.util.HashMap;

import org.apache.log4j.Logger;

import eu.clarin.cmdi.mdservice.internal.Cache;
import eu.clarin.cmdi.mdservice.model.Diagnostic;
import eu.clarin.cmdi.mdservice.model.Query;
import eu.clarin.cmdi.mdservice.model.WorkspaceProfile;
import eu.clarin.cmdi.mdservice.proxy.MDRepoProxy;
import eu.clarin.cmdi.mdservice.proxy.Pz2Proxy;
import eu.clarin.cmdi.mdservice.proxy.SRUProxy;

/**
 * A Struts2 controller, responsible for calls to repositories.
 * After 2011-09 rework Action and Proxy are decoupled.
 * So Action calls appropriate Proxy based upon the target repository type
 * 
 * responds to requests (collections, model, recordset) 
 * by dispatching the requests to appropriate Proxy and filling the inputStream with the result
 * It is possible to switch target-repository dynamically  (on every request). I.e. the client explicitly selects one of the repositories.
 *
 *  
 * @author vronk
 */
public class RepoAction extends GenericAction 
{

	private static final long serialVersionUID = 1L;
	private static Logger log = Logger.getLogger("RepoAction");
	
	protected URL base_url ;
	//Query query;
	
	@Override
	public String getFullFormat() {		
		return getActionkey() + "2" + getFormat();
	}

	/*
	@Override
	protected void setDefaultParams() {	
		
		
		// set defaults
		/*
		if ( params.get("version") == null){
			addParam("version","1.2");
		}
		if ( params.get("operation") == null){
			addParam("operation","searchRetrieve");
		}
		//q=query			
		if (getQ() != null) {
			addParam("q",getQ());
			addParam("query",getQ());
		} else {
			addParam("q",this.getParam("query"));
		}
		
		if ( params.get("startRecord") == null){
			addParam("startRecord","1");
		}
		if ( params.get("maximumRecords") == null){
			addParam("maximumRecords","50");
		}
		
		if ( params.get("x-cmd-maxdepth") == null){
			if (params.get("maxdepth") == null){
				addParam("maxdepth","2");
			} 
		} else {
			addParam("maxdepth",params.get("x-cmd-maxdepth")[0]);
		}
		
		if ( params.get("x-cmd-repository") == null){
			if (params.get("repository") == null){
				addParam("repository",WorkspaceProfile.getRepositoryByIndex(0));
			} 
		} else {
			addParam("repository",params.get("x-cmd-repository")[0]);
		}
		
		if ( params.get("x-cmd-cache") == null){
			if (params.get("cache") == null){
				addParam("cache",Cache.SKIP);
			} 
		} else {
			addParam("cache",params.get("x-cmd-cache")[0]);
		}
		
		if ( params.get("x-cmd-lang") == null){
			if (params.get("lang") == null){
				addParam("lang","en");
			} 
		} else {
			addParam("lang",params.get("x-cmd-lang")[0]);
		}
		
		if ( params.get("x-cmd-format") == null){
			if (params.get("format") == null){
				if (this.format == null) {
					addParam("format","xml");
				} else if (this.format.equals("")){
					addParam("format","xml");
				} else {
					addParam("format",this.format);
				}
			} 
		} else {
			addParam("format",params.get("x-cmd-format")[0]);
		}

		addParam("fullformat",getFullFormat());
		
	}
*/
	/**
	 * Based on the repository type - we create a typed Proxy, that will do the work.
	 * If the type is not recognized - that is a bad user error    
	 */
	public void setTargetProxy(){
						
			if (getRepository()==null) {
				getDiagnostics().Add(Diagnostic.MANDATORY_NOTSUPPLIED, "repository", "");
				return;
			}
			
			if (!WorkspaceProfile.repositoryExists(getRepository())){
				getDiagnostics().Add(Diagnostic.UNSUPPORTED_PARAMETERVALUE, "repository", "repository=" + getRepository() );
				return;
			}

			//Query query = new Query();
			switch (WorkspaceProfile.RepositoryType.toRepositoryType(WorkspaceProfile.getRepositoryType(getRepository())))
			{		
			    case PAZPAR: 
			    	setTargetProxy(new Pz2Proxy(new Query(getActionkey(), this.getParams())));
			    	return;
			    case SRU:
			    	setTargetProxy(new SRUProxy(new Query(getActionkey(), this.getParams())));
			    	return;
			    case MD:
			    	setTargetProxy(new MDRepoProxy(new Query(getActionkey(), this.getParams())));
			    	return;
			    default:
			    	//TODO error  = bad workspaceProfile configuration
			    	getDiagnostics().Add(Diagnostic.SYSTEM_ERROR, "Error in workspaceprofile configuration", "repository=" + getRepository() );
			}
	}

}