package eu.clarin.cmdi.mdservice.proxy;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Map;

import org.apache.log4j.Logger;
import eu.clarin.cmdi.mdservice.action.GenericAction;
import eu.clarin.cmdi.mdservice.internal.CQLParseException;
import eu.clarin.cmdi.mdservice.internal.NoStylesheetException;
import eu.clarin.cmdi.mdservice.internal.Utils;
import eu.clarin.cmdi.mdservice.model.Diagnostic;
import eu.clarin.cmdi.mdservice.model.WorkspaceProfile;


/**
 * Base class for implementation of proxies to target systems.
 * This is now (after rework 2011-09) decoupled from the called Action. 
 * 
 * Defines the methods used for retrieving the data 
 * and provides basic implementation where possible.
 *
 * Tries to get the base_uri from Config (mdservice.properties) based on action_key
 *  
 * @author vronk
 *
 */
public class BasicProxy implements ProxyInterface{

	private static final long serialVersionUID = 1L;
	private static Logger log = Logger.getLogger("BasicProxy");
 
	private String proxy_key = "basic";
	
	private GenericAction source_action;
	
	public GenericAction getSourceAction() {
			return source_action;
	}
	
	@Override
	public void setSourceAction(GenericAction action) {		
		source_action = action;
		
	}
	
	public void addParam (String key, String value){
		 getSourceAction().addParam(key, value);
	 }
	
	public Map<String,String[]> getParams() {
		return getSourceAction().getParams();
	}
	
	/**
	 * Convenience function to access the request-parameters in the Action  
	 * @param key
	 * @return
	 */
	public String getParam(String key) {
		return getSourceAction().getParam(key);
	}
	
	public void checkParams() {
		// required Params
/*
		//if (getParam("q").equals("")) {
		//	getSourceAction().getDiagnostics().Add(Diagnostic.MANDATORY_NOTSUPPLIED, "q");
		//}
		if (getParams().get("format") == null){
			if (getSourceAction().getFormat() == null) {
				addParam("format","xml");
			} else if (getSourceAction().getFormat().equals("")){
				addParam("format","xml");
			} else {
				addParam("format",getSourceAction().getFormat());
			}
		} 
		
		addParam("fullformat",getSourceAction().getFullFormat());
		*/
	}
	
	/**
	 * identification of the target-proxy (type)
	 * base for finding the right base_url in the props
	 * subclass has to override with its specific proxykey
	 * @return
	 */
	public String getProxyKey() {				
			return proxy_key;		
	}

	/**
	 * Get the base_uri based on the action-key
	 */
		
	public String getBaseURI() {		
		String uri =Utils.getConfig().getProperty(getSourceAction().getActionkey() + ".uri");	
		//String uri = Termset.getTermsetAttr(getSourceAction().getActionkey(), "url");
		return uri;
	}

	/**
	 * Provides the base-URL specific to given target repository or registry 
	 * @return 
	 * @throws MalformedURLException
	 */	
	public URL getBaseURL() throws MalformedURLException {		
		URL base_url = new URL(getBaseURI());		
		return base_url;
	}
	
	
	/**
	 * uses base_url + url_pattern (parametrized by actionkey) to form a url
	 * @return the full target-request URL to be sent to the target repository
	 * @throws CQLParseException 
	 * @throws Exception 
	 */	
	public URL getTargetRequest() throws IOException, CQLParseException {
		 
		URL targetURL =new URL(getBaseURI() + this.getSourceAction().getTargetRequestParams());
		
		log.debug("getTargetRequest().targetURL:" + targetURL);
	    return targetURL;
		
	}

	public InputStream getSourceStream() throws IOException, NoStylesheetException, CQLParseException {
		URLConnection urlConnection = getTargetRequest().openConnection();
		getSourceAction().setRequestProperties(urlConnection);
		return urlConnection.getInputStream();//.openStream();		
	}

	

}	