package eu.clarin.cmdi.mdservice.proxy;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;

import org.apache.log4j.Logger;

import eu.clarin.cmdi.mdservice.action.GenericAction;
import eu.clarin.cmdi.mdservice.internal.CQLParseException;
import eu.clarin.cmdi.mdservice.internal.NoStylesheetException;
import eu.clarin.cmdi.mdservice.model.Diagnostic;
import eu.clarin.cmdi.mdservice.model.Diagnostics;
import eu.clarin.cmdi.mdservice.model.Query;
import eu.clarin.cmdi.mdservice.model.WorkspaceProfile;

/**
 * Base class for implementation of proxies to target systems.
 * This is now (after rework 2011-09) decoupled from the called Action. 
 * 
 * Defines the methods used for retrieving the data 
 * and provides basic implementation where possible.
 *
 * In contrast to base class BasicProxy RepoProxy tries to get the base_uri from WorkspaceProfile (based on the repository-parameter)
 *  
 * @author vronk
 *
 */
public class RepoProxy extends BasicProxy{

	private static final long serialVersionUID = 1L;
	private static Logger log = Logger.getLogger("RepoProxy");
 
	private String proxy_key = "repo";
	private Diagnostics diagnostics;

	// should perhaps be RepoAction, but it gets complicated with the types
	private GenericAction source_action;
	
	private Query query;
	
	
	public RepoProxy(Query q){
		setQuery(q);
	}
	
	public GenericAction getSourceAction() {
			return source_action;
	}
	
/*	public RepoAction getSourceAction() {
		return source_action;
}*/
	
	@Override
	public void setSourceAction(GenericAction action) {		
		source_action = action;
		
	}
	
	public Diagnostics getDiagnostics(){
		return this.diagnostics;
	}
	
	public void setDiagnostics(Diagnostics diagnostics){
		this.diagnostics = diagnostics;
	}
	
	/**
	 * Convenience function to access the request-parameters in the Action  
	 * @param key
	 * @return
	 */
	/* This is already defined in BasicProxy! ?
	 public String getParam(String key) {
	 
		return getSourceAction().getParam(key);
	}
	*/
	
	
	public void checkParams() {
		// required Params
		//super.checkParams();
		// check and set defaults
		if (WorkspaceProfile.getRepositoryPath(getParam("repository")) == null) {
			getSourceAction().getDiagnostics().Add(Diagnostic.UNSUPPORTED_PARAMETERVALUE, "repository");
		}
		if (getParam("repository")=="") {
			getSourceAction().getDiagnostics().Add(Diagnostic.MANDATORY_NOTSUPPLIED, "repository");
		}
	}
	
	
	/**
	 * identification of the target-proxy (type)
	 * base for finding the right base_url in the props
	 * subclass has to override with its specific proxykey
	 * @return
	 */
	public String getProxyKey() {				
			return proxy_key;		
	}

	/**
	 * Override of the basic method provided by the super-class:
	 * Read the repository-uri from the WorkspaceProfile based on the repository-parameter 
	 */
		
	public String getBaseURI() {		
		String uri = WorkspaceProfile.getRepositoryPath(getSourceAction().getRepository());		
		return uri;
	}

	public Diagnostics Diagnostics(){
		if (this.diagnostics == null){
			this.diagnostics = new Diagnostics();
		}
		return this.diagnostics;
	}
	/**
	 * Provides the base-URL specific to given target repository or registry 
	 * @return
	 * @throws MalformedURLException
	 */	
	public URL getBaseURL() throws MalformedURLException {		
		URL base_url = new URL(getBaseURI());		
		return base_url;
	}
	
	
	public InputStream getSourceStream() throws IOException, NoStylesheetException, CQLParseException {
		return getTargetRequest().openStream();		
	}
 
	 public String fullQueryString(){
		String  full_query_string= "TODO: construct fullQueryString";
		
		/* matej 20110903
		 *  String full_query_string= this.getQ();
		 	
		if (!getSquery().equals("") && !this.getQ().equals("")){
				full_query_string = "(" + this.getSquery() + " ) and (" + this.getQ() + ")";
			} else if (!this.getSquery().equals("")) {
				full_query_string = this.getSquery();
			} else if (!this.getQ().equals("")){
				full_query_string = this.getQ();
			} else {
				full_query_string = "";
			}
		*/
		return full_query_string;
	 }
	 

	/**
	 * uses base_url + url_pattern (parametrized by actionkey) to form a url
	 * @return the full target-request URL to be sent to the target repository
	 * @throws CQLParseException 
	 * @throws Exception 
	 */	
	public URL getTargetRequest() throws IOException, CQLParseException {
		 
		
		//URL targetURL =new URL( getBaseURL(), urls.get(getActionkey()) + query.toURLParam() );
		URL targetURL =getBaseURL();
		
		log.debug("getTargetRequest().targetURL:" + targetURL);
	    return targetURL;
		
	}

	public void setQuery(Query query) {
		this.query = query;
	}

	public Query getQuery() {
		return query;
	}

	

}	